package com.galaxis.manatee.dao;

import com.galaxis.manatee.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSubWeekHourDO;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Repository
public interface ChuanyunSubWeekHourDao extends GalaxisRepository<ChuanyunSubWeekHourDO,String> {

    /**
     * 获取周工时
     * @param userId
     * @return
     */
    @Query(value="SELECT chuanyun_self_sub_work_hour.object_id, \n" +
            "            LEFT(YEARWEEK(chuanyun_self_sub_work_hour.day_log_date,1),4)as year, \n" +
            "            RIGHT(YEARWEEK(chuanyun_self_sub_work_hour.day_log_date,1),2) AS week, \n" +
            "            project_id,user_id,bg, \n" +
            "            SUM(standard_work_hour)AS standard_work_hour, \n" +
            "            project_type,\n" +
            "            `status` \n" +
            "            FROM chuanyun_self_sub_work_hour  \n" +
            "            WHERE project_id !='' and user_id=?1\n" +
            "            GROUP BY chuanyun_self_sub_work_hour.project_id, \n" +
            "            chuanyun_self_sub_work_hour.project_type, \n" +
            "            chuanyun_self_sub_work_hour.user_id,\n" +
            "            LEFT(YEARWEEK(chuanyun_self_sub_work_hour.day_log_date,1),4), \n" +
            "            RIGHT(YEARWEEK(chuanyun_self_sub_work_hour.day_log_date,1),2), \n" +
            "            chuanyun_self_sub_work_hour.`status`",nativeQuery = true)
    List<ChuanyunSubWeekHourDO> getSubWeekHourByUserId(@Param("userId") String userId);

    /**
     * 获取原有周工时数据
     * @param userId
     * @param projectId
     * @param projectType
     * @param week
     * @param year
     * @param status
     * @return
     */
    ChuanyunSubWeekHourDO findByUserIdAndProjectIdAndProjectTypeAndWeekAndYearAndStatus(String userId,String projectId,String projectType, String week,String year,String status);

    /**
     * 查询是否有流程中数据
     * @param projectId
     * @param userId
     * @param year
     * @param week
     * @param projectType
     * @return
     */
    @Query(value="SELECT chuanyun_self_sub_work_hour.object_id,\n" +
            "            DATE_FORMAT(day_log_date,'%Y') as year,\n" +
            "            WEEKOFYEAR(chuanyun_self_sub_work_hour.day_log_date) AS week,\n" +
            "            project_id,user_id,\n" +
            "            SUM(standard_work_hour)AS standard_work_hour,\n" +
            "            chuanyun_self_sub_work_hour.bg,\n" +
            "            project_type,\n" +
            "            `status`\n" +
            "            FROM chuanyun_self_sub_work_hour \n" +
            "            WHERE project_id=?1 and user_id=?2 and DATE_FORMAT(day_log_date,'%Y')=?3 and \n" +
            "            WEEKOFYEAR(chuanyun_self_sub_work_hour.day_log_date)=?4 and status=2 and project_type=?5\n" +
            "             GROUP BY chuanyun_self_sub_work_hour.project_id,\n" +
            "            chuanyun_self_sub_work_hour.project_type,\n" +
            "            chuanyun_self_sub_work_hour.user_id,\n" +
            "            DATE_FORMAT(day_log_date,'%Y'),\n" +
            "            WEEKOFYEAR(chuanyun_self_sub_work_hour.day_log_date),\n" +
            "            chuanyun_self_sub_work_hour.`status`",nativeQuery = true)
    ChuanyunSubWeekHourDO getProcessSubWeekHour(@Param("projectId") String projectId, @Param("userId") String userId, @Param("year") String year, @Param("week") String week, @Param("projectType") String projectType);

    /**
     * 删除处于流程中的数据
     * @param projectId
     * @param userId
     * @param year
     * @param week
     * @param status
     * @param projectType
     */
    @Transactional(rollbackFor = Exception.class)
    void deleteByProjectIdAndUserIdAndYearAndWeekAndStatusAndProjectType(String projectId, String userId, String year, String week, String status, String projectType);

    /**
     * 根据用户Id获取周工时
     * @param userId
     * @return
     */
    List<ChuanyunSubWeekHourDO> findByUserId(String userId);

}
